﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net;
using Newtonsoft.Json.Linq;

namespace TextMarksV2APIClient_DotNet
{    
    public class TextMarksV2APIClient
    {
        // -----------------------------------------------------------------------
        // Configuration:
        private const string API_URL_BASE = "https://api2ssl.textmarks.com";
        // ---

        public TextMarksV2APIClient(string sApiKey, string sAuthUser, string sAuthPass)
        {
            _sApiKey = sApiKey;
            _sAuthUser = sAuthUser;
            _sAuthPass = sAuthPass;
        }

        public TextMarksV2APIClient()
        {
        }

        public void SetApiKey(string sApiKey) { _sApiKey = sApiKey; }
        public void SetAuthUser(string sAuthUser) { _sAuthUser = sAuthUser; }
        public void SetAuthPass(string sAuthPass) { _sAuthPass = sAuthPass; }

        public string GetApiKey() { return _sApiKey; }
        public string GetAuthUser() { return _sAuthUser; }
        public string GetAuthPass() { return _sAuthPass; }

        /// <summary>
        /// Public method to call API.
        /// 
        /// The API Key and auth params are automatically added if present.
        /// </summary>
        /// <param name="sPackageName"></param> Package name.
        /// <param name="sMethodName"></param> Method name.
        /// <param name="msoParams"></param> Params for method.
        /// <returns></returns> Decoded (from JSON) response.
        public JObject Call(string sPackageName, string sMethodName, Dictionary<string, object> msoParams)
        {
            return CallJsonApiMethod(sPackageName, sMethodName, msoParams);
        }

        /// <summary>
        /// Execute HTTP request (post params to API endpoint) and return string response.
        /// </summary>
        /// <param name="sUrl"></param> URL to request (method endpoint).
        /// <param name="msoParams"></param> Request params.
        /// <returns></returns> Response (usually JSON string).
        protected string RawHttpCall(String sUrl, Dictionary<string, object> msoParams)
        {
            // Convert param map to encoded form (to post):
            string sPostData = string.Empty;

            try
            {
                StringBuilder sbPostData = new StringBuilder();

                foreach (var msoParam in msoParams)
                {
                    sbPostData.Append("&").Append(msoParam.Key);
                    sbPostData.Append("=").Append(msoParam.Value.ToString());
                }

                sPostData = sbPostData.ToString();
            }
            catch (Exception)
            {
                throw;
            }

            // Create HTTP URL connection:
            HttpWebRequest request;

            Uri uri = new Uri(sUrl);
            request = (HttpWebRequest)WebRequest.Create(uri);

            try
            {
                request.Method = "POST";
                request.ContentLength = sPostData.Length;
                request.ContentType = "application/x-www-form-urlencoded";

            }
            catch (Exception e)
            {
                throw new TextMarksV2APIClientTransportException("TextMarksV2APIClient (" + sUrl + ") saw HTTP connection error: " + e.ToString(), e);
            }

            try
            {
                using (Stream writeStream = request.GetRequestStream())
                {
                    UTF8Encoding encoding = new UTF8Encoding();
                    byte[] bytes = encoding.GetBytes(sPostData);
                    writeStream.Write(bytes, 0, bytes.Length);
                }
            }
            catch (Exception e)
            {
                throw new TextMarksV2APIClientTransportException("TextMarksV2APIClient (" + sUrl + ") saw HTTP request error: " + e.ToString(), e);
            }

            // Get response:
            try
            {
                string result = string.Empty;

                using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                {
                    using (Stream responseStream = response.GetResponseStream())
                    {
                        using (StreamReader readStream = new StreamReader(responseStream, Encoding.UTF8))
                        {
                            result = readStream.ReadToEnd();
                        }
                    }
                }
                return result;
            }
            catch (Exception e)
            {
                throw new TextMarksV2APIClientTransportException("TextMarksV2APIClient (" + sUrl + ") saw HTTP response error: " + e.ToString(), e);
            }
        }
        
        /// <summary>
        /// Execute API call and return decoded JSON response.
        /// 
        /// The API Key and auth params are automatically added.
        /// </summary>
        /// <param name="sPackageName"></param> Package name.
        /// <param name="sMethodName"></param> Method name.
        /// <param name="msoParams"></param> Params for method.
        /// <returns></returns> Decoded (from JSON) response.
        protected JObject CallJsonApiMethod(string sPackageName, string sMethodName, Dictionary<string, object> msoParams)
        {
            // Prep:
            Dictionary<string, object> msoParamsFull = new Dictionary<string, object>(msoParams); // (copy to keep original clean)
            if (_sApiKey != null) { msoParamsFull.Add("api_key", _sApiKey); }
            if (_sAuthUser != null) { msoParamsFull.Add("auth_user", _sAuthUser); }
            if (_sAuthPass != null) { msoParamsFull.Add("auth_pass", _sAuthPass); }
            string sUrl = API_URL_BASE + "/" + sPackageName + "/" + sMethodName + "/";

            // Make actual HTTP call:            
            string sResp = RawHttpCall(sUrl, msoParamsFull);

            // Parse JSON response:
            JObject joResp = null;

            try
            {
                joResp = JObject.Parse(sResp);
            }
            catch (Exception ex)
            {
                throw new TextMarksV2APIClientTransportException("TextMarksV2APIClient (" + sUrl + ") got invalid JSON response: " + ex.Message, ex);
            }

            // Check API response code:
            try
            {
                int iResCode = int.Parse(joResp["head"]["rescode"].ToString());
                string sResMsg = joResp["head"]["resmsg"].ToString();

                if (iResCode != 0)
                {
                    throw new TextMarksV2APIClientResultException("TextMarksV2APIClient.call("
                      + sPackageName + "." + sMethodName + ") got API error #" + iResCode + ": " + sResMsg, iResCode);
                }
            }
            catch (Exception e)
            {
                throw new TextMarksV2APIClientTransportException("TextMarksV2APIClient (" + sUrl + ") got bad malformed JSON response: " + e.ToString(), e);
            }

            return joResp;
        }

        // -----------------------------------------------------------------------
        private string _sApiKey = null;
        private string _sAuthUser = null;
        private string _sAuthPass = null;
        // -----------------------------------------------------------------------
    }
}
